function [klr,BOOLsuccess]=COMkerLRpre(K,C,y)
  tic;
  problem=setupKLR(K,C,y);
  [x,fval,exitflag,out,lambda,grad]=fmincon(problem);
  if (exitflag>0),
    BOOLsuccess=true;
    klr=getklrparms(K,C,y,x,fval,toc,out,grad);
  else, klr=[], BOOLsuccess=false; out, out.message, keyboard; end;
end

function problem=setupKLR(K,C,y);
  global probdata;
  posind=find(y==+1);
  negind=find(y==-1);
  probdata=struct('K',K,'C',C,'y',y,'posind',posind,'negind',negind);

  [NUMpts,NUMpts]=size(K);
  problem.objective=@klrprimal;
  problem.x0=zeros(NUMpts+1,1);
  problem.Aineq=[];
  problem.bineq=[];
  problem.Aeq=[];
  problem.beq=[];
  problem.lb=-Inf(NUMpts+1,1);
  problem.ub=[];
  problem.nonlcon=[];
  problem.solver='fmincon';
  problem.options=optimset('Diagnostic','off','Display','off','Algorithm','interior-point','GradObj','on','Hessian','lbfgs','DerivativeCheck','off');
end

function [f,g]=klrprimal(x)
  global probdata;

  b=x(end);
  alphay=x(1:end-1);
  Kalphay=probdata.K*alphay;
  predict=b+Kalphay;
  predict(probdata.negind)=-predict(probdata.negind);

  xi=1-predict;
  nlind=find(xi<40);
  numlikelihood=Inf(size(xi));numlikelihood(nlind)=exp(xi(nlind));
  loss=xi;loss(nlind)=log(1+numlikelihood(nlind));
  f=double(0.5*(alphay'*Kalphay)+probdata.C*sum(loss));
  
  if (nargout>1)
    beta=repmat(probdata.C,size(xi));
    beta(nlind)=probdata.C*(numlikelihood(nlind)./(1+numlikelihood(nlind)));
    beta(probdata.posind)=-beta(probdata.posind);
    g=[probdata.K*(alphay+beta);sum(beta)];
  end;
end

function klr=getklrparms(K,C,y,x,primalobj,traintime,out,grad)
  klr=[];
  klr.b=x(end);
  klr.alphay=x(1:end-1);
  klr.primalobj=primalobj;
  klr.trainerr=100*mean(y~=sign(klr.b+K*klr.alphay));
  klr.time=traintime;
  
  klr.diagnostic.gradmag=sqrt(grad'*grad);
  klr.diagnostic.gradmax=max(abs(grad));
  klr.diagnostic.NUMiter=out.iterations;
  klr.diagnostic.NUMfeval=out.funcCount;
  klr.diagnostic.NUMcg=out.cgiterations;
  klr.diagnostic.NUMcviolations=out.constrviolation;
  klr.diagnostic.stepsize=out.stepsize;
  klr.diagnostic.MSGexit=out.message;
end