function [svm,BOOLsuccess]=COMkerSVMpre(K,C,labels)
% Function to compute a binary SVM
%
% [svm,BOOLsuccess]=SKLlearn2class(K,C,labels)
% 
% Inputs:
%   K      - NUMpts x NUMpts square matrix of precomputed kernel values. It
%            is your responsibility to make sure that min(eig(K))>0.
%   C      - scalar misclassification penalty. You can modify this code to
%            have C1 and C2 so as to handle unbalanced data.
%   labels - NUMpts x 1 vector of class labels in {-1, +1}
% 
% Outputs:
%   svm         - A structure containing the SVM solution
%   BOOLsuccess - Boolean indicating whether the optimisation
%                 terminated successfully or not.

  [NUMpts,NUMpts]=size(K);
  Y=diag(labels);
  H=Y*K*Y;
  svm=[];

  tic;
  f=-ones(NUMpts,1);      % -\sigma{\alpha_{i}} in Wolfe dual
  Aeq=labels';            % Constraint \sigma{\alpha_{i} labels_{i}} == 0
  beq=0;
  LB=zeros(NUMpts,1);     % Lower Bound: \alpha_{i} >= 0
  UB=repmat(C,NUMpts,1);  % Upper Bound: \alpha_{i} <= C
  X0=zeros(NUMpts,1);     % Starting point set to 0
  OPTIONS=optimset('Display','off','LargeScale','off','MaxIter',2000);
  [alpha,minval,exitflag,output]=quadprog(H,f,[],[],Aeq,beq,LB,UB,X0,OPTIONS);
  deltat=toc;
  
  EPS=1e-5;
  BOOLsuccess=1;
  svind=find(alpha>EPS);
  if ((length(svind)>=2) & (exitflag>0)),
    margin=2/sqrt(alpha'*H*alpha);
    bind=find((alpha>EPS) & (alpha<C-EPS));
    if (~isempty(bind)),
      b=mean(labels(bind).*(1-H(bind,:)*alpha));
      alpha=alpha(svind);
      trainerr=100*sum(labels'~=sign(b+alpha'*diag(labels(svind))*K(svind,:)))/length(labels);
      labels=labels(svind);
      svm=struct('alpha',alpha,'b',b,'labels',labels,'svind',svind,'margin',margin,...
	  'trainerr',trainerr,'primalobj',-minval,'C',C,'time',deltat);
    end;
  else
    BOOLsuccess=0;
    output.message
  end;
end