function K=COMkernelize(FEAtrain,FEAtest,parms,BOOLmakePD)
%
% Function to compute a kernel given feature vectors
%
% K=COMkernelize(FEAtrain,FEAtest,parms,BOOLmakePD)
%
% Inputs;
%   FEAtrain   - NUMdims x NUMtrain matrix.
%   FEAtest    - NUMdims x NUMtest matrix
%   parms      - Kernel parameters. Look at the code to figure out the
%                parameters for your specific kernel.
%   BOOLmakePD - Should the kernel be made positive definite.
%
% Outputs:
%   K          - NUMtrain x NUMtest kernel matrix.

  [NUMdims,NUMtest]=size(FEAtest);
  [NUMdims,NUMtrain]=size(FEAtrain);
  
  if     (strcmpi(parms.name,'linear')), K=FEAtrain'*FEAtest/NUMdims;
  elseif (strcmpi(parms.name,'poly')),   K=(FEAtrain'*FEAtest/NUMdims+parms.bias).^parms.deg;
  elseif (strcmpi(parms.name,'echi2')),  K=exp(-parms.gamma*pdistchisqr(FEAtrain,FEAtest));
  elseif (strcmpi(parms.name,'rbf')),    K=exp(-parms.gamma*pdisteucsqr(FEAtrain,FEAtest));
  else, fprintf('Unknown kernel type.\n'); K=[]; end;
  
  if ((BOOLmakePD) && (~isempty(K))), K=K+(1e-4-min([eig(K);0]))*eye(size(K)); end;
end

function d=pdisteucsqr(x1,x2)
  [NUMdims,NUMx1]=size(x1);
  [NUMdims,NUMx2]=size(x2);
  d=zeros(NUMx1,NUMx2);
  for i=1:NUMx1, d(i,:)=sum((x2-repmat(x1(:,i),1,NUMx2)).^2,1); end;
end

function d=pdistchisqr(x1,x2)
  [NUMdims,NUMx1]=size(x1);
  [NUMdims,NUMx2]=size(x2);
  d=zeros(NUMx1,NUMx2);
  for i=1:NUMx1, 
    tmp=repmat(x1(:,i),1,NUMx2);
    num=(x2-tmp).^2;
    denom=x2+tmp;
    denom(find(denom==0))=1;
    d(i,:)=0.5*sum(num./denom); 
  end;
end