function [lr,BOOLsuccess]=COMlinLR(X,C,y)
%
% Function to learn a regularized linear binary logistic regression
% classifier. This code does not enforce large margin constraints,
% i.e. y*f(x) >= 1. To enforce that use COMkerLRpre which is based
% on kernels and then recover w. Use COMkerLRpre in any case if you
% want to use kernels. For multinomial LR use COMlinLRmulti which 
% is also linear and does not enforce large margins.

  tic;
  problem=setupLR(X,C,y);
  [x,fval,exitflag,out,lambda,grad]=fmincon(problem);
  if (exitflag>0),
    BOOLsuccess=true;
    lr=getlrparms(X,C,y,x,fval,toc,out,grad);
  else, lr=[], BOOLsuccess=false; out, out.message, keyboard; end;
end

function problem=setupLR(X,C,y);
  global probdata;
  posind=find(y==+1);
  negind=find(y==-1);
  probdata=struct('X',X,'C',C,'posind',posind,'negind',negind);

  [NUMdims,NUMpts]=size(X);
  problem.objective=@lrprimal;
  problem.x0=zeros(NUMdims+1,1);
  problem.Aineq=[];
  problem.bineq=[];
  problem.Aeq=[];
  problem.beq=[];
  problem.lb=-Inf(NUMdims+1,1);
  problem.ub=[];
  problem.nonlcon=[];
  problem.solver='fmincon';
  problem.options=optimset('Diagnostic','off','Display','off','Algorithm','interior-point','GradObj','on','Hessian','lbfgs','DerivativeCheck','off');
end

function [f,g]=lrprimal(x)
  global probdata;

  b=x(end);
  w=x(1:end-1);
  predict=(b+w'*probdata.X)';
  predict(probdata.posind)=-predict(probdata.posind);

  nlind=find(predict<40);
  numlikelihood=Inf(size(predict));
  numlikelihood(nlind)=exp(predict(nlind));
  loss=predict;loss(nlind)=log(1+numlikelihood(nlind));
  f=double(0.5*(w'*w)+probdata.C*sum(loss));
  
  if (nargout>1)
    beta=ones(size(predict));
    beta(nlind)=numlikelihood(nlind)./(1+numlikelihood(nlind));
    beta(probdata.posind)=-beta(probdata.posind);
    expect=sum(repmat(beta',length(w),1).*probdata.X,2);
    g=[w+probdata.C*expect;probdata.C*sum(beta)];
  end;
end

function lr=getlrparms(X,C,y,x,primalobj,traintime,out,grad)
  lr=[];
  lr.b=x(end);
  lr.w=x(1:end-1);
  lr.primalobj=primalobj;
  lr.trainerr=100*mean(y~=sign(lr.b+lr.w'*X)');
  lr.time=traintime;
  
  lr.diagnostic.gradmag=sqrt(grad'*grad);
  lr.diagnostic.gradmax=max(abs(grad));
  lr.diagnostic.NUMiter=out.iterations;
  lr.diagnostic.NUMfeval=out.funcCount;
  lr.diagnostic.NUMcg=out.cgiterations;
  lr.diagnostic.NUMcviolations=out.constrviolation;
  lr.diagnostic.stepsize=out.stepsize;
  lr.diagnostic.MSGexit=out.message;
end