function [mlr,BOOLsuccess]=COMlinLRmulti(X,C,y)
  tic;
  problem=setupMLR(X,C,y);
  [x,fval,exitflag,out,lambda,grad]=fmincon(problem);
  if (exitflag>0),
    BOOLsuccess=true;
    mlr=getmlrparms(X,C,y,x,fval,toc,out,grad);
  else, mlr=[], BOOLsuccess=false; out, out.message, keyboard; end;
end

function problem=setupMLR(X,C,y);
  global probdata;

  probdata=[];
  probdata.X=X;
  probdata.C=C;
  ClassIDs=unique(y);
  probdata.NUMclasses=length(ClassIDs);
  [probdata.NUMdims,probdata.NUMpts]=size(X);
  probdata.NUMclasspts=zeros(probdata.NUMclasses,1);
  for k=1:probdata.NUMclasses,
    probdata.classinfo(k).ind=find(y==ClassIDs(k));
    probdata.classinfo(k).ClassID=ClassIDs(k);
  probdata.classinfo(k).sumx=sum(X(:,probdata.classinfo(k).ind),2);
    probdata.NUMclasspts(k)=length(probdata.classinfo(k).ind);
  end;

  NUMvars=(probdata.NUMdims+1)*probdata.NUMclasses;
  problem.objective=@mlrprimal;
  problem.x0=zeros(NUMvars,1);
  problem.Aineq=[];
  problem.bineq=[];
  problem.Aeq=[];
  problem.beq=[];
  problem.lb=-Inf(NUMvars,1);
  problem.ub=[];
  problem.nonlcon=[];
  problem.solver='fmincon';
  problem.options=optimset('Diagnostic','off','Display','off','Algorithm','interior-point','GradObj','on','Hessian','lbfgs','DerivativeCheck','off');
end

function [f,g]=mlrprimal(x)
  global probdata;

  b=x(probdata.NUMdims*probdata.NUMclasses+1:end);
  w=x(1:probdata.NUMdims*probdata.NUMclasses);
  f=0.5*(w'*w);
  w=reshape(w,probdata.NUMdims,probdata.NUMclasses);
  
  predict=w'*probdata.X+repmat(b,1,probdata.NUMpts);
  expredict=exp(predict);
  z=sum(expredict);
    
  nll=sum(log(z))-b'*probdata.NUMclasspts;
  for k=1:probdata.NUMclasses,
    nll=nll-w(:,k)'*probdata.classinfo(k).sumx;
  end;
  f=f+probdata.C*nll;
  
  if (nargout>1)
    g=zeros(size(x));
    p=expredict./repmat(z,probdata.NUMclasses,1);
    for k=1:probdata.NUMclasses,
      expect=sum(repmat(p(k,:),probdata.NUMdims,1).*probdata.X,2);
      gk=w(:,k)+probdata.C*(expect-probdata.classinfo(k).sumx);
      g((k-1)*probdata.NUMdims+1:k*probdata.NUMdims)=gk;
    end;
    gb=probdata.C*(sum(p,2)-probdata.NUMclasspts);
    g(probdata.NUMdims*probdata.NUMclasses+1:end)=gb;
  end;
end

function mlr=getmlrparms(X,C,y,x,primalobj,traintime,out,grad)
  ClassIDs=unique(y);
  [NUMdims,NUMpts]=size(X);
  NUMclasses=length(ClassIDs);

  mlr=[];
  mlr.time=traintime;
  mlr.ClassIDs=ClassIDs;
  mlr.primalobj=primalobj;
  mlr.b=x(NUMdims*NUMclasses+1:end);
  mlr.w=reshape(x(1:NUMdims*NUMclasses),NUMdims,NUMclasses);
  predict=mlr.w'*X+repmat(mlr.b,1,NUMpts);
  [maxval,maxind]=max(predict);
  mlr.trainerr=100*mean(y~=ClassIDs(maxind));
  
  mlr.diagnostic.gradmag=sqrt(grad'*grad);
  mlr.diagnostic.gradmax=max(abs(grad));
  mlr.diagnostic.NUMiter=out.iterations;
  mlr.diagnostic.NUMfeval=out.funcCount;
  mlr.diagnostic.NUMcg=out.cgiterations;
  mlr.diagnostic.NUMcviolations=out.constrviolation;
  mlr.diagnostic.stepsize=out.stepsize;
  mlr.diagnostic.MSGexit=out.message;
end